from builtins import object
import platform

def get_exclusive_lock(name):
    """ Lock 

    :param name: lock name
    :type name: string

    :return: lock file path
    :rtype: string
    """
    if platform.system()=='Windows':
        import portalocker
        fd = open("/tmp/{}.lock".format(name), "w")
        portalocker.lock(fd, portalocker.LOCK_EX)
        return fd
    else:
        import fcntl
        fd = open("/tmp/{}.lock".format(name), "w")
        fcntl.lockf(fd.fileno(), fcntl.LOCK_EX)
        return fd

def unlock_exclusive_lock(fd):
    """ Unlock

    :param fd: lock file path
    :type fd: string
    """
    if platform.system()=='Windows':
        import portalocker
        portalocker.unlock(fd)
        fd.close()
    else:
        import fcntl
        fcntl.lockf(fd.fileno(), fcntl.LOCK_UN)

class ExclusiveLock(object):
    """ Exclusive Lock class"""
    def __init__(self, name):
        """Constructor
        
        :param name: lock name
        :type name: string"""
        self.__name = name
    def __enter__(self):
        """ Activate lock

        :return: lock tmp file
        :rtype: string
        """
        self.__fd = get_exclusive_lock(self.__name)
        return self.__fd
    def __exit__(self, exc_type, exc_value, traceback):
        """ Unlock, protected method
        """
        unlock_exclusive_lock(self.__fd)
